﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using Unity.MLAgents;
using Unity.MLAgents.Sensors;
using Unity.MLAgents.Actuators;

public class AgentImitation2 : Agent
{
    [Header("Speed")]
    [Range(0f, 5f)]
    public float _speed = 2f;

    [Header("Turn speed")]
    [Range(50f, 300f)]
    public float _turnSpeed = 50f;

    public bool _training = true;


    protected Rigidbody _rb;

    [SerializeField]
    protected Target[] _target;
    protected Animator _anim;

    protected Vector3 _previous;

    public Text _contadorText = null;
    public float _puntos = 0;

    float[] _oldstateTargets = new float[3];
    GameObject[] walls;

    public override void Initialize()
    {
        _rb = GetComponent<Rigidbody>();
        _anim = GetComponent<Animator>();
        _previous = transform.position;

        walls = GameObject.FindGameObjectsWithTag("wall");
        //MaxStep part Agent class
        if (!_training) MaxStep = 0;
    }

    public override void OnEpisodeBegin()
    {
        _rb.velocity = Vector3.zero;
        _rb.angularVelocity = Vector3.zero;

        //NewPosition();
        _previous = transform.position;

        if (walls != null)
        {
            foreach (GameObject wall in walls)
            {
                //NewPosition(wall.transform, 5f, 0.01f); 
            }
        }
        
    }

    public override void OnActionReceived(ActionBuffers actions)
    {
        float lForward = actions.DiscreteActions[0];
        float lTurn = 0;
        if (actions.DiscreteActions[1] == 1)
        {
            lTurn = -1;
        }
        else if (actions.DiscreteActions[1] == 2)
        {
            lTurn = 1;
        }
        _rb.MovePosition(transform.position +
            transform.forward * lForward * _speed * Time.deltaTime);
        transform.Rotate(transform.up * lTurn * _turnSpeed * Time.deltaTime);
        AddReward(-1f / MaxStep);
    }

    public virtual void Update()
    {
        float velocity = ((transform.position - _previous).magnitude) / Time.deltaTime;
        _previous = transform.position;

        _anim.SetFloat("multiplicador", velocity);
    }


    public override void CollectObservations(VectorSensor sensor1)
    {

        for (int n = 0; n < _target.Length; n++)
        {

            if (_target[n].isLigthActive()) 
            {
                sensor1.AddObservation(
                    Vector3.Distance(_target[n].transform.position, transform.position));
                sensor1.AddObservation(
                    (_target[n].transform.position - transform.position).normalized);
            }            
        }

        sensor1.AddObservation(
            transform.forward);

    }

    public override void Heuristic(in ActionBuffers actionsOut)
    {     
        var discreteActionsOut = actionsOut.DiscreteActions;
        discreteActionsOut.Clear();

        int lForward = 0;
        int lTurn = 0;
        if (Input.GetKey(KeyCode.UpArrow))
        {
            lForward = 1;
        }
        if (Input.GetKey(KeyCode.LeftArrow))
        {
            lTurn = 1;
        }
        else if (Input.GetKey(KeyCode.RightArrow))
        {
            lTurn = 2;
        }
        discreteActionsOut[0] = lForward;
        discreteActionsOut[1] = lTurn;
    }


    private void OnTriggerStay(Collider other)
    {
        if (true)
        {
            
            if (other.CompareTag("target"))
            {
                if (other.GetComponent<Target>().isLigthActive())
                {
                    GiveReward(1.5f);
                }
                else
                {
                    //GiveReward(-0.5f);
                }
                    
            }
            if (other.CompareTag("borders") || other.CompareTag("wall"))
            {
                GiveReward(-0.05f);
            }
        }
    }

    protected void GiveReward(float premio)
    {
        if (_training)
        {
            AddReward(premio);
        }

        if (_contadorText != null)
        {
            _puntos += premio;
            _contadorText.text = _puntos.ToString();
        }
    }

    protected void NewPosition(Transform pT = null, float pRange = 3, float pCollision = 0.5f)
    {
        bool possiblePosition = false;
        int intentos = 100;
        Vector3 potentialPosition = Vector3.zero;
        if (pT == null) pT = transform; 

        while (!possiblePosition || intentos >= 0)
        {
            intentos--;

            potentialPosition = new Vector3(
                pT.parent.position.x + UnityEngine.Random.Range(-pRange, pRange),
                0.555f,
                pT.parent.position.z + UnityEngine.Random.Range(-pRange, pRange));
            //If we have colliders in the array we have a collision
            Collider[] colliders = Physics.OverlapSphere(potentialPosition, pCollision);
            if (colliders.Length == 0 || intentos <= 1)
            {
                pT.position = potentialPosition;
                possiblePosition = true;
            }
        }
    }
}
